//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
import ij.IJ;
import ij.ImagePlus;
import ij.Macro;
import ij.WindowManager;
import ij.plugin.PlugIn;
import spotcaliper.detector.OutputMode;
import spotcaliper.detector.Parameters;
import spotcaliper.detector.RunningMode;
import spotcaliper.gui.DialogDetection;

public class SpotCaliper_Plain implements PlugIn {

	@Override
	public void run(String arg) {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp == null) {
			IJ.error("No open image.");
			return;
		}
		int type = imp.getType();
		if (type != ImagePlus.GRAY8 && type != ImagePlus.GRAY16 && type != ImagePlus.GRAY32) {
			IJ.error("Only process 8-bit, 16-bit or 32-bit image.");
			return;
		}
		
		if (Macro.getOptions() == null) {
			Parameters params = new Parameters(true);
			new DialogDetection(imp, RunningMode.PLAIN, OutputMode.SAVE, params);
		}
		else {
			Parameters params = new Parameters(true);
			params.setAutomaticValues(imp);
			params.fromMacro(Macro.getOptions());
			new DialogDetection(imp, RunningMode.MACRO, OutputMode.SAVE, params);
		}
	}
}
