//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package scalablewavelets;

public class Multipliers {

	static double	sqrt2	= Math.sqrt(2.0);
	static double PILOG2	= Math.PI / Math.log(2.0);
	
	public static double[] channelIn1DArray(double R[], double[] alpha, double rho, int nx, int ny) {	
		int n = R.length;
		double[] y = new double[n];
		for (int k = 0; k < n; ++k) {
			y[k] = channelprofile( R[k] / rho, alpha);
		}
		return y;
	}

	public static double channelprofile(double x, double[] alpha) {
		int lmax = alpha.length - 1;
		int nmax = 2 * lmax + 1;
		double y = alpha[0];
		for (int l = 1; l <= lmax; ++l) {
			y += sqrt2 * alpha[l] * Math.cos(PILOG2 * l * Math.log(x));
		}
		y = y / Math.sqrt(nmax);
		return y;
	}
}
