package spotcaliper.detector;

import java.text.SimpleDateFormat;
import java.util.Calendar;

import spotcaliper.gui.components.InfoPane;
import ij.ImagePlus;
import ij.Macro;

public class Parameters {

	private boolean		brightSpots		= true;
	public double		radiusMin		= 2;
	public double		radiusMax		= 40;
	public double		overlap			= 0;
	public int			margin			= 0;
	public double		delta			= 100.0;
	public int			nSpots			= Integer.MAX_VALUE;

	public String		filename		= "";
	public double		contrastMinimum	= 100;

	public boolean		showAnalysis	= false;
	public boolean		multithread		= true;

	public Parameters(boolean brightSpots) {
		this.brightSpots = brightSpots;
	}

	public boolean getBrightSpots() {
		return brightSpots;
	}

	public void setBrightSpots(boolean brightSpots) {
		this.brightSpots = brightSpots;
	}

	public void setAutomaticValues(ImagePlus imp) {
		int nx = imp.getWidth();
		int ny = imp.getHeight();
		radiusMax = (int)(Math.sqrt(nx * nx + ny * ny) * 0.05);
		radiusMin = 4;
		overlap = 0;
		delta = 100;
		nSpots = 2 * (int) Math.ceil(imp.getWidth() * imp.getHeight() / (2.0 * radiusMin * radiusMin * Math.PI)) + 1;
		margin = (int) Math.ceil(radiusMax);
		showAnalysis = false;
		multithread = true;
		contrastMinimum = imp.getStatistics().stdDev * 0.5;
	}

	public void fromMacro(String options) {
		brightSpots = Macro.getValue(options, "spots", "bright").equals("bright");
		radiusMin = Double.parseDouble(Macro.getValue(options, "rmin", "" + radiusMin));
		radiusMax = Double.parseDouble(Macro.getValue(options, "rmax", "" + radiusMax));
		overlap = Double.parseDouble(Macro.getValue(options, "overlap", "" + overlap));
		margin = (int)Double.parseDouble(Macro.getValue(options, "margin", "" + margin));
		nSpots = (int)Double.parseDouble(Macro.getValue(options, "nspots", ""+ nSpots));
		filename = Macro.getValue(options, "output", ""+ filename);
	}
	
	public String toMacro() {
		String options = "";
		options += "spots=" + (brightSpots ? "bright" : "dark") + " ";
		options += "rmin=" + radiusMin + " ";
		options += "rmax=" + radiusMax + " ";
		options += "overlap=" + overlap + " ";
		options += "margin=" + margin + " ";
		options += "nspots=" + nSpots + " ";
		options += "output=[" + filename + "] ";
		return options;
	}
	
	public void setInformation(ImagePlus imp, InfoPane info) {
		info.append("Image title: " + imp.getTitle());
		info.append("Image size: " + imp.getWidth() + "x" + imp.getHeight());
		info.append("Spots: " + (brightSpots ? "Bright spots" : "Dark spots"));
		info.append("Minimum Radius: " + radiusMin + " pixels");
		info.append("Maximum Radius: " + radiusMax + " pixels");
		info.append("Admissible overlap: " + overlap + " pixels");
		info.append("Margin border: " + margin + " pixels");
		info.append("Max number of spots: " + nSpots);
		info.append("Minimum contrast: " + contrastMinimum);
		info.append("Accuracy (delta): " + delta);
		info.append("Output: " + filename);
		info.append("Show Analyzed Images: " + showAnalysis);
		info.append("Multithreading: " + multithread);
	}

	@Override
	public String toString() {
		return "SpotCaliper run: " + new SimpleDateFormat("dd MMMM yyyy HH:mm:ss").format(Calendar.getInstance().getTime()) + "\n" + "Spots: "
				+ (brightSpots ? "Bright spots / dark background" : "Dark spots / bright background") + "\n" + "Mininum Radius: " + radiusMin + " pixels" + "\n" + "Maxinum Radius: " + radiusMax
				+ " pixels" + "\n" + "Admissible overlap: " + overlap + " pixels" + "\n" + "Margin border: " + margin + " pixels" + "\n" + "Upper bound for the number of spots: " + nSpots + "\n"
				+ "\n" + "Accuracy (delta): " + delta + "\n" + filename + "\n" + "Show Analyzed Images: " + showAnalysis + "\n" + "Multithreading: "
				+ multithread + "\n";

	}
}
