package spotcaliper.gui;

import ij.IJ;
import ij.ImagePlus;
import ij.gui.GUI;
import ij.gui.ImageWindow;
import ij.plugin.frame.Recorder;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import scalablewavelets.Analysis;
import spotcaliper.detector.OutputMode;
import spotcaliper.detector.Parameters;
import spotcaliper.detector.RunningMode;
import spotcaliper.detector.Spot;
import spotcaliper.detector.SpotDetector;
import spotcaliper.gui.components.GridPanel;
import spotcaliper.gui.components.GridToolbar;
import spotcaliper.gui.components.InfoPane;
import spotcaliper.gui.components.ProgressionBar;
import spotcaliper.gui.components.SpinnerDouble;
import spotcaliper.gui.components.SpinnerInteger;
import spotcaliper.gui.components.TitlePane;
import spotcaliper.gui.settings.Settings;

public class DialogDetection extends JDialog implements Runnable, ActionListener, ChangeListener {

	private Settings		settings			= new Settings("SpotCaliper", IJ.getDirectory("plugins") + "SpotCaliper.txt");
	private String			copyright			= "(c) 2015 BIG EPFL - Version 2.0";
	private Data			data;
	private SpotDetector	detector;
	private Parameters		params;

	private JButton			bnStop				= new JButton("Stop");
	private JButton			bnHelp				= new JButton("Help");
	private JButton			bnHelp2				= new JButton("Help");
	private JButton			bnBrowse			= new JButton("Browse");

	private JButton			bnCancel			= new JButton("Cancel");
	private JButton			bnRun				= new JButton("Run");

	private ProgressionBar	progress			= new ProgressionBar(copyright);
	private InfoPane		info				= new InfoPane();
	private ImagePlus		imp;

	private SpinnerDouble	spnRadiusMin		= new SpinnerDouble(3, 1, 1000, 1);
	private SpinnerDouble	spnRadiusMax		= new SpinnerDouble(30, 1, 1000, 1);
	private SpinnerDouble	spnDelta			= new SpinnerDouble(100, 1, 10000, 1);
	private SpinnerDouble	spnContrastMinimum	= new SpinnerDouble(10, -9999999, 999999999, 1);
	private SpinnerDouble	spnOverlap			= new SpinnerDouble(0, -99999, 99999, 1);
	private SpinnerInteger	spnNumberMax		= new SpinnerInteger(1000, 1, 999999, 1);
	private SpinnerInteger	spnMargin			= new SpinnerInteger(10, 0, 99999, 1);
	private JComboBox		cmbSpots			= new JComboBox(new String[] { "Bright Spots", "Dark Spots" });
	private JComboBox		cmbSpotsLoad		= new JComboBox(new String[] { "Bright Spots", "Dark Spots" });

	private JCheckBox		chkMultithread		= new JCheckBox("Multithread");
	private JCheckBox		chkAnalyzedImage	= new JCheckBox("Show analysis");
	private JLabel			lblScale			= new JLabel("XX scales / XX channels");

	private RunningMode		mode				= RunningMode.STANDARD;
	private boolean			stop				= false;

	private JRadioButton	chkSelection		= new JRadioButton("Out in the selection tool", true);
	private JRadioButton	chkSave				= new JRadioButton("Out in a CSV file");
	private JRadioButton	chkAutomatic		= new JRadioButton("Automatic");
	private JRadioButton	chkCustom			= new JRadioButton("Custom", true);
	private JRadioButton	chkLoad				= new JRadioButton("Load");
	private JTextField		txtFilename;
	private GridPanel		pnParameters		= new GridPanel("Parameters", 4);
	private GridPanel 		pnSave 				= new GridPanel("Output file", 4);

	private JLabel			lblSpots			= new JLabel("Spots");
	private JPanel			cards				= new JPanel(new CardLayout());
	private JPanel			cardsInner			= new JPanel(new CardLayout());

	public DialogDetection(ImagePlus imp, RunningMode mode, OutputMode output, Parameters params) {
		super(new JFrame(), "SpotCaliper " + (mode == RunningMode.PLAIN ? "Plain" : ""));
		this.data = new Data(imp);
		this.detector = new SpotDetector(imp, progress);
		this.imp = imp;
		this.params = params;
		this.mode = mode;
		lblScale.setBorder(BorderFactory.createEtchedBorder());
		txtFilename = new JTextField("filename", 15);
		txtFilename.setText(System.getProperty("user.home") + File.separator + imp.getTitle() + ".csv");
		
		chkSave.setSelected(output == OutputMode.SAVE);
		chkSelection.setSelected(output == OutputMode.SELECTION);
		
		JToolBar status = new JToolBar("Status");
		status.setLayout(new BorderLayout(0, 0));
		status.setBorder(BorderFactory.createLineBorder(new Color(30, 30, 30, 60)));
		status.setFloatable(false);
		status.add(bnHelp, BorderLayout.WEST);
		status.add(progress, BorderLayout.CENTER);
		status.add(bnStop, BorderLayout.EAST);

		JPanel pnJournal = new JPanel(new BorderLayout());
		pnJournal.add(info.getScrollPane(), BorderLayout.CENTER);
		pnJournal.add(status, BorderLayout.SOUTH);
		cards.add(pnJournal, "journal");

		ButtonGroup group1 = new ButtonGroup();
		group1.add(chkAutomatic);
		group1.add(chkCustom);
		group1.add(chkLoad);

		ButtonGroup group2 = new ButtonGroup();
		group2.add(chkSelection);
		group2.add(chkSave);

		settings.record("cmbSpots", cmbSpots, "Bright Spots");
		settings.record("cmbSpotsLoad", cmbSpotsLoad, "Bright Spots");
		settings.record("spnDelta", spnDelta, "100");
		settings.record("spnRadiusMin", spnRadiusMin, "2");
		settings.record("spnRadiusMax", spnRadiusMax, "30");
		settings.record("spnOverlap", spnOverlap, "0");
		settings.record("spnNumberMax", spnNumberMax, "1000");
		settings.record("spnMargin", spnMargin, "10");
		settings.record("chkMultithread", chkMultithread, true);
		settings.record("chkAnalyzedImage", chkAnalyzedImage, false);
		settings.record("spnContrastMinimum", spnContrastMinimum, "10");
		settings.loadRecordedItems();

		// Panel Parameters
		pnParameters.place(0, 0, lblSpots);
		pnParameters.place(0, 1, cmbSpots);
		pnParameters.place(1, 0, "Radius Min");
		pnParameters.place(1, 1, spnRadiusMin);
		pnParameters.place(1, 2, "px");
		pnParameters.place(2, 0, "Radius Max");
		pnParameters.place(2, 1, spnRadiusMax);
		pnParameters.place(2, 2, "px");
		pnParameters.place(3, 0, "Overlap");
		pnParameters.place(3, 1, spnOverlap);
		pnParameters.place(3, 2, "px");
		pnParameters.place(4, 0, "Margin");
		pnParameters.place(4, 1, spnMargin);
		pnParameters.place(4, 2, "px");
		pnParameters.place(5, 0, "Max. number of spots");
		pnParameters.place(5, 1, spnNumberMax);
		pnParameters.place(6, 0, "Minimum Contrast");
		pnParameters.place(6, 1, spnContrastMinimum);

		if (mode == RunningMode.DEVELOPER) {
			pnParameters.place(8, 0, "<html>Accuracy</html>");
			pnParameters.place(8, 1, spnDelta);
			pnParameters.place(8, 2, "<html>&delta;</html>");
			pnParameters.place(9, 0, "<html>Complexity</html>");
			pnParameters.place(9, 1, 2, 1, lblScale);
			pnParameters.place(10, 0, chkMultithread);
			pnParameters.place(10, 1, chkAnalyzedImage);
		}

		GridPanel pnLoad = new GridPanel(true, 8);
		pnLoad.place(0, 0, "Spot");
		pnLoad.place(0, 1, cmbSpotsLoad);
		pnLoad.place(1, 0, 2, 1, txtFilename);
		pnLoad.place(2, 1, 1, 1, bnBrowse);
		cardsInner.add(pnLoad, "load");


		GridPanel pnDetection = new GridPanel(false, 4);
		pnDetection.place(0, 0, 3, 1, new TitlePane(copyright));

		GridPanel pnMode = new GridPanel(false, 4);
		pnMode.place(0, 0, chkAutomatic);
		pnMode.place(0, 1, chkCustom);
		if (mode == RunningMode.DEVELOPER || mode == RunningMode.STANDARD) {
			pnMode.place(0, 2, chkLoad);
		}
		
		if (mode == RunningMode.UNIQUE) {
			GridPanel pn = new GridPanel(false, 4);
			pn.place(1, 0, 1, 1, chkSelection);
			pn.place(1, 1, 1, 1, chkSave);
			pnDetection.place(3, 0, 3, 1, pn);
		}
	
		if (mode == RunningMode.PLAIN || mode == RunningMode.UNIQUE) {
			pnSave.place(1, 0, 1, 1, txtFilename);
			pnSave.place(1, 1, 1, 1, bnBrowse);
			pnDetection.place(4, 0, 3, 1, pnSave);
		}
		
		if (mode == RunningMode.LOAD) {
			chkLoad.setSelected(true);
		}
		else {
			cardsInner.add(pnParameters, "parameters");
			pnDetection.place(1, 0, 3, 1, pnMode);
		}

		pnDetection.place(2, 0, 3, 1, cardsInner);
		pnDetection.place(5, 0, bnHelp2);
		pnDetection.place(5, 1, bnCancel);
		pnDetection.place(5, 2, bnRun);

		bnHelp2.addActionListener(this);
		bnCancel.addActionListener(this);
		bnRun.addActionListener(this);
		cards.add(pnDetection, "detector");

		setLayout(new BorderLayout());
		add(cards);

		setCard(mode == RunningMode.MACRO ? "journal" : "detector");
		spnRadiusMax.addChangeListener(this);
		bnStop.addActionListener(this);
		bnHelp.addActionListener(this);
		bnBrowse.addActionListener(this);
		chkAutomatic.addActionListener(this);
		chkCustom.addActionListener(this);
		chkLoad.addActionListener(this);
		chkSelection.addActionListener(this);
		chkSave.addActionListener(this);
		
		pack();
		updateInterface();

		Dimension scr = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension dlg = this.getSize();
		if (imp != null) {
			ImageWindow win = imp.getWindow();
			if (win != null) {
				Dimension img = win.getSize();
				if (img != null) {
					Point loc = imp.getWindow().getLocation();
					if (loc.x + img.width + dlg.width < scr.width)
						setLocation(loc.x + img.width + 5, loc.y);
				}
			}
			else
				GUI.center(this);
		}

		setVisible(true);
		setModal(false);

		if (mode == RunningMode.MACRO) {
			Thread thread = new Thread(this);
			thread.setPriority(Thread.MIN_PRIORITY);
			thread.start();
		}
	}

	private void setCard(String name) {
		CardLayout cl = (CardLayout) (cards.getLayout());
		cl.show(cards, name);
	}

	@Override
	public void actionPerformed(ActionEvent event) {

		if (event.getSource() == bnCancel)
			dispose();
		else if (event.getSource() == bnStop)
			stop();
		else if (event.getSource() == bnBrowse)
			browse();
		else if (event.getSource() == bnHelp || event.getSource() == bnHelp2)
			WebBrowser.open("http://bigwww.epfl.ch/algorithms/spotcaliper/");
		else if (event.getSource() == chkAutomatic)
			updateInterface();
		else if (event.getSource() == chkCustom)
			updateInterface();
		else if (event.getSource() == chkLoad)
			updateInterface();
		else if (event.getSource() == chkSelection)
			updateInterface();
		else if (event.getSource() == chkSave)
			updateInterface();

		else if (event.getSource() == bnRun) {
			if (imp == null) {
				progress.setString("Invalid source image.");
				return;
			}
			if (imp.getProcessor() == null) {
				progress.setString("Invalid source image.");
				return;
			}
			Thread thread = new Thread(this);
			thread.setPriority(Thread.MIN_PRIORITY);
			thread.start();
			settings.storeRecordedItems();
		}
	}

	@Override
	public void stateChanged(ChangeEvent event) {
		if (event.getSource() == spnRadiusMax) {
			lblScale.setText("" + Analysis.getNumberOfScales(spnRadiusMax.get()) + " scales / " + Analysis.getNumberOfChannel() + " channels");
		}
	}

	@Override
	public void run() {

		setCard("journal");

		if (chkLoad.isSelected()) {
			data.loadCVS(txtFilename.getText());
			params.radiusMax = imp.getWidth() + imp.getHeight();
			for (Spot spot : data.getDetected())
				if (spot.r > params.radiusMax)
					params.radiusMax = spot.r;
			info.append("" + data.getDetected().size() + " spots loaded");
		}

		info.appendSeparator();

		setParameters();

		progress.reset("Spot Detector ");
		double chrono = System.nanoTime();

		double map[][] = detector.analysis(params);
		info.appendSeparator();
		info.append("Time: " + new SimpleDateFormat("dd MMMM yyyy HH:mm:ss").format(Calendar.getInstance().getTime()));
		info.appendSeparator();
		if (!chkLoad.isSelected()) {
			detector.detectAndSteer(map, data, params);
			progress.progress(data.getDetected().size() + " spots detected", 100);
			info.append("" + data.getLocalMax().size() + " local maxima");
			info.append("" + data.getDetected().size() + " spots detected");
			info.append("" + data.getSelected().size() + " spots selected");
		}
		else {
		}

		int mb = 1024 * 1024;
		Runtime instance = Runtime.getRuntime();
		info.append("Total memory: " + instance.totalMemory() / mb + "Mb");
		info.append("Used memory: " + (instance.totalMemory() - instance.freeMemory()) / mb + "Mb");
		info.append("Elapsed time: " + (System.nanoTime() - chrono) / (1000000.0) + "ms");
		if (stop)
			return;

		dispose();

		if (chkSelection.isSelected())
			new DialogSelection(imp, detector, data, info, mode==RunningMode.DEVELOPER);
		if (chkSave.isSelected())
			data.saveCVS(params.filename);
	}

	private void setParameters() {

		if (mode != RunningMode.MACRO) {
			if (chkLoad.isSelected())
				params.setBrightSpots(cmbSpotsLoad.getSelectedIndex() == 0);
			else
				params.setBrightSpots(cmbSpots.getSelectedIndex() == 0);
			params.radiusMax = Math.max(spnRadiusMin.get(), spnRadiusMax.get());
			params.radiusMin = Math.min(spnRadiusMin.get(), spnRadiusMax.get());
			params.overlap = spnOverlap.get();
			params.margin = spnMargin.get();
			params.nSpots = spnNumberMax.get();
			params.contrastMinimum = spnContrastMinimum.get();
			params.delta = 100.0;
			params.showAnalysis = false;
			params.multithread = true;
			params.filename = txtFilename.getText();
		}

		if (mode == RunningMode.DEVELOPER) {
			params.showAnalysis = chkAnalyzedImage.isSelected();
			params.multithread = chkMultithread.isSelected();
			params.delta = spnDelta.get();
		}

		params.setInformation(imp, info);

		if (Recorder.record)
			Recorder.record("run", "SpotCaliper ", params.toMacro());
	}

	private void updateInterface() {
		
		if (chkSelection.isSelected()) {
			Component cs[] = pnSave.getComponents();
			for (Component c : cs)
				c.setEnabled(false);
			pnSave.setVisible(false);
			pack();
		}
		if (chkSave.isSelected()) {
			Component cs[] = pnSave.getComponents();
			for (Component c : cs)
				c.setEnabled(true);
			pnSave.setVisible(true);
			pack();
		}
		
		CardLayout cl = (CardLayout) (cardsInner.getLayout());
		cl.show(cardsInner, chkLoad.isSelected() ? "load" : "parameters");

		Component cs[] = pnParameters.getComponents();

		if (chkAutomatic.isSelected()) {
			params.setAutomaticValues(imp);
			spnRadiusMin.set(params.radiusMin);
			spnRadiusMax.set(params.radiusMax);
			spnOverlap.set(params.overlap);
			spnMargin.set(params.margin);
			spnNumberMax.set(params.nSpots);
			spnContrastMinimum.set(params.contrastMinimum);
			for (Component c : cs)
				c.setEnabled(false);
		}

		if (chkCustom.isSelected()) {
			settings.loadRecordedItems();
			for (Component c : cs)
				c.setEnabled(true);
		}

		lblSpots.setEnabled(true);
		cmbSpots.setEnabled(true);

	}

	private void stop() {
		stop = true;
		detector.stop();
		if (mode == RunningMode.DEVELOPER || mode == RunningMode.STANDARD)
			setCard("detector");
		else
			dispose();
	}

	private void browse() {
		JFileChooser saver = new JFileChooser();
		if (mode == RunningMode.PLAIN) {
			saver.setDialogTitle("Specify the filename to save");
			saver.setSelectedFile(new File(imp.getTitle() + ".csv"));
			int ret = saver.showSaveDialog(new JFrame());
			if (ret == JFileChooser.APPROVE_OPTION)
				txtFilename.setText(saver.getSelectedFile().getAbsolutePath());
		}
		else {
			saver.setDialogTitle("Specify the filename to load");
			saver.setSelectedFile(new File(imp.getTitle() + ".csv"));
			int ret = saver.showOpenDialog(new JFrame());
			if (ret == JFileChooser.APPROVE_OPTION)
				txtFilename.setText(saver.getSelectedFile().getAbsolutePath());
		}
	}

}
