//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.gui.components;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

public class InfoPane extends JEditorPane {

	public InfoPane() {
		Color color = new JPanel().getBackground();
		int r = color.getRed();
		int g = color.getRed();
		int b = color.getRed();
		setBorder(BorderFactory.createEtchedBorder());
		String html = "";
		html += "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n";
		html += "<html><head>\n";
		html += "<style>body {font-family:verdana arial;}</style>\n";
		html += "<style>h1 {text-align:center; font-size:1.3em; color:#333; font-weight:bold; padding:1px; margin:2px;}</style>\n";
		html += "<style>h2 {text-align:center; font-size:1.0em; font-weight:italic; padding:1px; margin:2px;}</style>\n";
		html += "<style>h3 {text-align:center; font-size:1.0em; font-weight:bold; padding:4px; margin:2px;}</style>\n";
		html += "<style>p {font-size:1em; padding:1px; margin:1px;}</style>\n";
		html += "</head>\n";
		html += "<body style=\"margin:15px;background-color: rgb(" + r + "," + g + "," + b + ")\">\n";
		html += "<h1>SpotCaliper</h1>";
		html += "<p>Fast Wavelet-based Spot Detection with Accurate Size Estimation</p>";
		html += "<p><i>Biomedical Imaging Group, EPFL</i></p>";
		html += "<p style=\"font-size:0.85em;\">http://bigwww.epfl.ch/algorithms/spotcaliper/</p>";
		html += "</body></html>\n";
		setEditable(false);
		setContentType("text/html; charset=ISO-8859-1");
		setText(html);
	}

	public JScrollPane getScrollPane() {
		JScrollPane scrollInfo = new JScrollPane(this);
		scrollInfo.setPreferredSize(new Dimension(350, 400));
		return scrollInfo;
	}
	
	public void appendSeparator() {
		append("<center>&bull;&bull;&bull;</center>");
	}
	
	public void append(String s) {
		try {
			HTMLEditorKit kit = new HTMLEditorKit();
		    HTMLDocument doc = (HTMLDocument)getDocument();
		    setEditorKit(kit);
		    setDocument(doc);
		    kit.insertHTML(doc, doc.getLength(), "<p>"+s+"</p>", 0, 0, null);
		    setCaretPosition(doc.getLength() - 1);
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}
