//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.gui.components;

import javax.swing.JProgressBar;

public class ProgressionBar extends JProgressBar {

	private double chrono;
	
	public ProgressionBar(String msg) {
		setStringPainted(true);
		reset(msg);
	}

	public void progress(String msg, double value) {
		progress(msg, (int)value);
	}

	public void progress(String msg, int value) {
		double elapsedTime = System.currentTimeMillis() - chrono;
		String t = " [" + (elapsedTime > 3000 ?  Math.round(elapsedTime/10)/100.0 + "s." : elapsedTime + "ms") + "]";
		setValue(value);
		setString(msg + t);
	}
	
	public void reset(String msg) {
		chrono = System.currentTimeMillis();
		setValue(0);
		setString(msg );
	}
}
