//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.gui.components;

import java.util.ArrayList;

import javax.swing.JEditorPane;

/**
 * This class extends the Java JEditorPane to make a easy to use panel to
 * display HTML information.
 * 
 * @author Daniel Sage, Biomedical Imaging Group, EPFL, Lausanne, Switzerland.
 * 
 */
public class TextPane extends JEditorPane {

	public TextPane(String content) {
		String html = "";
		html += "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n";
		html += "<html><head>\n";
		html += "<style>body {font-family:verdana arial;}</style>\n";
		html += "<style>h1 {text-align:center; font-size:1.0em; font-weight:bold; padding:1px; margin:2px;}</style>\n";
		html += "<style>h2 {text-align:center; font-size:1.0em; font-weight:italic; padding:1px; margin:2px;}</style>\n";
		html += "<style>h3 {text-align:center; font-size:1.0em; font-weight:bold; padding:1px; margin:2px;}</style>\n";
		html += "<style>p {font-size:1em; padding:1px; margin:2px;}</style>\n";

		html += "</head>\n";
		html += "<body>\n";
		html += content;
		html += "</body></html>\n";

		setEditable(false);
		setContentType("text/html; charset=ISO-8859-1");
		setText(html);
	}
	
	public TextPane(ArrayList<String> contents) {
		String html = "";
		html += "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n";
		html += "<html><head>\n";
		html += "<style>body {font-family:verdana arial;}</style>\n";
		html += "<style>h1 {text-align:center; font-size:1.0em; font-weight:bold; padding:1px; margin:2px;}</style>\n";
		html += "<style>h2 {text-align:center; font-size:1.0em; font-weight:italic; padding:1px; margin:2px;}</style>\n";
		html += "<style>h3 {text-align:center; font-size:1.0em; font-weight:bold; padding:1px; margin:2px;}</style>\n";
		html += "<style>p {font-size:1em; padding:1px; margin:2px;}</style>\n";

		html += "</head>\n";
		html += "<body>\n";
		for(String s : contents)	
			html += "<p>" + s + "</p>";
		html += "</body></html>\n";

		setEditable(false);
		setContentType("text/html; charset=ISO-8859-1");
		setText(html);
	}

}
