//====================================================================================================
// Project: SpotCaliper
// 
// Authors: Zsuzsanna Puspoki and Daniel Sage
// Organization: Biomedical Imaging Group (BIG), Ecole Polytechnique Federale de Lausanne
// Address: EPFL-STI-IMT-LIB, 1015 Lausanne, Switzerland
//
// Information: http://bigwww.epfl.ch/algorithms/spotcaliper/
//
// References:
// Zsuzsanna Puspoki et al.
// SpotCaliper: Fast Wavelet-based Spot Detection with Accurate Size Estimation 
// Bioinformatics Oxford, submitted in June 2015.
// Available at: http://bigwww.epfl.ch/publications/
//
// Conditions of use:
// You'll be free to use this software for research purposes, but you should not redistribute 
// it without our consent. In addition, we expect you to include a citation whenever you 
// present or publish results that are based on it.
//====================================================================================================
package spotcaliper.gui.components;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.text.DecimalFormat;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class TickLabelSlider extends JSlider implements ChangeListener, MouseListener, MouseMotionListener {

	private JLabel		lbl		= new JLabel("050");
	private JPopupMenu	info	= new JPopupMenu();
	private JMenuItem	value	= new JMenuItem();

	public TickLabelSlider() {
		setPaintLabels(true);
		Hashtable<Integer, JLabel> table = new Hashtable<Integer, JLabel>();
		table.put(0, new JLabel("Sensitive"));
		table.put(100, new JLabel("Fast"));
		setLabelTable(table);
		lbl.setBorder(BorderFactory.createEtchedBorder());
		this.addChangeListener(this);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.setMinimumSize(new Dimension(200, 50));
		value.setFont(new Font(value.getFont().getFamily(), Font.PLAIN, value.getFont().getSize() - 2));
		info.add(value);
	}

	@Override
	public void stateChanged(ChangeEvent event) {
		JSlider slider = (JSlider) event.getSource();
		DecimalFormat df = new DecimalFormat("000");
		lbl.setText(df.format(slider.getValue()));
		setToolTipText(new Integer(getValue()).toString());
		value.setText(slider.getValue() + "");
	}

	public JLabel getLabel() {
		return lbl;
	}

	@Override
	public void mousePressed(MouseEvent e) {
		info.setVisible(true);
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		info.show(e.getComponent(), e.getX() + 10, e.getY() + 10);
	}

	@Override
	public void mouseMoved(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		info.setVisible(false);
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mouseClicked(MouseEvent e) {
	}

}
